"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Generator = void 0;
const assert_1 = __importDefault(require("assert"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const openapi_codegen_utils_1 = require("@fresha/openapi-codegen-utils");
const getResourceType = (name, schema) => {
    let typeSchema;
    if (schema.allOf?.length) {
        typeSchema = schema.allOf?.find(item => item.getProperty('type'));
        if (typeSchema) {
            typeSchema = typeSchema.getProperty('type');
        }
    }
    if (!typeSchema) {
        typeSchema = schema.getProperty('type');
    }
    (0, assert_1.default)(typeSchema, `Cannot determine JSON:API resource type for schema name ${name}`);
    const result = typeSchema.enum?.at(0);
    (0, assert_1.default)(typeof result === 'string', `JSON:API resource type must be a string (name=${name})`);
    return result;
};
class Generator extends openapi_codegen_utils_1.Generator {
    constructor(context) {
        super(context);
        this.schemaToName = new Map();
        this.schemaToType = new Map();
        this.resourceUsage = new Map();
    }
    collectData() {
        for (const [name, schema] of this.context.openapi.components.schemas) {
            if (name.endsWith('Resource')) {
                const type = getResourceType(name, schema);
                this.schemaToName.set(schema, name);
                this.schemaToType.set(schema, type);
            }
        }
        const mediaType = (0, openapi_codegen_utils_1.getMediaType)(true);
        for (const operation of (0, openapi_codegen_utils_1.getOperations)(this.context.openapi)) {
            // const requestBodySchema = operation.requestBody?.content.get(mediaType)?.schema;
            // if (requestBodySchema && requestBodySchema.type === 'object') {
            //   const primaryData = requestBodySchema.getPropertyOrThrow('data');
            //   if (schemaToName.has(primaryData)) {
            //   }
            //   // const included = requestBodySchema.getProperty('included');
            //   // if (included) {
            //   // }
            // }
            for (const response of operation.responses.codes.values()) {
                const responseDocumentSchema = response.content.get(mediaType)?.schema;
                let primaryDataSchema = responseDocumentSchema?.getProperty('data') ?? null;
                if (primaryDataSchema?.type === 'array') {
                    primaryDataSchema = primaryDataSchema.items;
                }
                if (primaryDataSchema?.anyOf?.length) {
                    for (const subschema of primaryDataSchema.anyOf) {
                        this.addUsage(subschema, operation.parent.pathUrl, operation.httpMethod, undefined, 'data');
                    }
                }
                else if (primaryDataSchema) {
                    this.addUsage(primaryDataSchema, operation.parent.pathUrl, operation.httpMethod, undefined, 'data');
                }
                const includedSchema = responseDocumentSchema?.getProperty('included')?.items ?? null;
                const subschemas = [];
                if (includedSchema?.anyOf?.length) {
                    subschemas.push(...includedSchema.anyOf);
                }
                if (includedSchema?.oneOf?.length) {
                    subschemas.push(...includedSchema.oneOf);
                }
                if (subschemas.length) {
                    for (const subschema of subschemas) {
                        this.addUsage(subschema, operation.parent.pathUrl, operation.httpMethod, undefined, 'include');
                    }
                }
                else if (includedSchema) {
                    this.addUsage(includedSchema, operation.parent.pathUrl, operation.httpMethod, undefined, 'include');
                }
            }
        }
    }
    addUsage(schema, url, httpMethod, inRequest, inResponse) {
        const resourceName = this.schemaToName.get(schema);
        if (!resourceName) {
            global.console.log(schema);
        }
        (0, assert_1.default)(resourceName, `Something is wrong at ${url}, ${httpMethod}, ${schema.title ?? ''}`);
        const entries = this.resourceUsage.get(resourceName);
        if (entries) {
            entries.push({ url, httpMethod, inRequest, inResponse });
        }
        else {
            this.resourceUsage.set(resourceName, [{ url, httpMethod, inRequest, inResponse }]);
        }
    }
    generateCode() {
        const outputPath = path_1.default.join(this.context.outputPath, 'resource-usage.html');
        const rows = [];
        for (const [resourceName, usages] of this.resourceUsage) {
            for (const usage of usages) {
                let inResponse = '';
                if (usage.inResponse === 'data') {
                    inResponse = 'DATA';
                }
                else if (usage.inResponse === 'include') {
                    inResponse = 'INCLUDE';
                }
                rows.push(`
          <tr>
            <td>${resourceName}</td>
            <td>${usage.httpMethod.toUpperCase()} ${usage.url}</td>
            <td>${usage.inRequest ? 'YES' : ''}</td>
            <td>${inResponse}</td>
          </tr>
        `);
            }
        }
        if (!this.context.dryRun) {
            const text = `
        <!DOCTYPE html>
        <html>
          <head>
            <style>
              td, th {
                border: black 1px solid;
                padding: 4px 15px;
              }
            </style>
          </head>
          <body>
            <table>
              <caption>Resource usage in ${this.context.openapi.info.title}</caption>
              <thead>
                <tr>
                  <th>Resource</th>
                  <th>Operation</th>
                  <th>Request</th>
                  <th>Response</th>
                </tr>
              </thead>
              <tbody>
                ${rows.join('\n')}
              </tbody>
            </table>
          </body>
        </html>
      `;
            fs_1.default.writeFileSync(outputPath, text, 'utf-8');
        }
    }
}
exports.Generator = Generator;
//# sourceMappingURL=Generator.js.map